package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.LocalDeclaration;
import org.eclipse.jdt.internal.compiler.ast.NullAnnotationMatching;
import org.eclipse.jdt.internal.compiler.ast.Statement;
import org.eclipse.jdt.internal.compiler.codegen.BranchLabel;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.flow.LoopingFlowContext;
import org.eclipse.jdt.internal.compiler.flow.UnconditionalFlowInfo;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.VariableBinding;

public class ForeachStatement extends Statement {
  public LocalDeclaration elementVariable;
  
  public int elementVariableImplicitWidening = -1;
  
  public Expression collection;
  
  public Statement action;
  
  private int kind;
  
  private static final int ARRAY = 0;
  
  private static final int RAW_ITERABLE = 1;
  
  private static final int GENERIC_ITERABLE = 2;
  
  private TypeBinding iteratorReceiverType;
  
  private TypeBinding collectionElementType;
  
  private BranchLabel breakLabel;
  
  private BranchLabel continueLabel;
  
  public BlockScope scope;
  
  public LocalVariableBinding indexVariable;
  
  public LocalVariableBinding collectionVariable;
  
  public LocalVariableBinding maxVariable;
  
  private static final char[] SecretIteratorVariableName = " iterator".toCharArray();
  
  private static final char[] SecretIndexVariableName = " index".toCharArray();
  
  private static final char[] SecretCollectionVariableName = " collection".toCharArray();
  
  private static final char[] SecretMaxVariableName = " max".toCharArray();
  
  int postCollectionInitStateIndex = -1;
  
  int mergedInitStateIndex = -1;
  
  public ForeachStatement(LocalDeclaration elementVariable, int start) {
    this.elementVariable = elementVariable;
    this.sourceStart = start;
    this.kind = -1;
  }
  
  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
    FlowInfo exitBranch;
    this.breakLabel = new BranchLabel();
    this.continueLabel = new BranchLabel();
    int initialComplaintLevel = ((flowInfo.reachMode() & 0x3) != 0) ? 1 : 0;
    this.collection.checkNPE(currentScope, flowContext, flowInfo, 1);
    flowInfo = this.elementVariable.analyseCode(this.scope, flowContext, flowInfo);
    FlowInfo condInfo = this.collection.analyseCode(this.scope, flowContext, flowInfo.copy());
    LocalVariableBinding elementVarBinding = this.elementVariable.binding;
    condInfo.markAsDefinitelyAssigned(elementVarBinding);
    this.postCollectionInitStateIndex = currentScope.methodScope().recordInitializationStates(condInfo);
    LoopingFlowContext loopingContext = 
      new LoopingFlowContext(flowContext, flowInfo, (ASTNode)this, this.breakLabel, 
        this.continueLabel, (Scope)this.scope, true);
    UnconditionalFlowInfo actionInfo = 
      condInfo.nullInfoLessUnconditionalCopy();
    actionInfo.markAsDefinitelyUnknown(elementVarBinding);
    if ((currentScope.compilerOptions()).isAnnotationBasedNullAnalysisEnabled) {
      int elementNullStatus = FlowInfo.tagBitsToNullStatus(this.collectionElementType.tagBits);
      int nullStatus = NullAnnotationMatching.checkAssignment(currentScope, flowContext, (VariableBinding)elementVarBinding, null, 
          elementNullStatus, this.collection, this.collectionElementType);
      if ((elementVarBinding.type.tagBits & 0x2L) == 0L)
        actionInfo.markNullStatus(elementVarBinding, nullStatus); 
    } 
    if (this.action != null && (!this.action.isEmptyBlock() || 
      (currentScope.compilerOptions()).complianceLevel > 3080192L)) {
      if (this.action.complainIfUnreachable((FlowInfo)actionInfo, this.scope, initialComplaintLevel, true) < 2)
        actionInfo = this.action.analyseCode(this.scope, (FlowContext)loopingContext, (FlowInfo)actionInfo).unconditionalCopy(); 
      exitBranch = flowInfo.unconditionalCopy()
        .addInitializationsFrom(condInfo.initsWhenFalse());
      if ((actionInfo.tagBits & loopingContext.initsOnContinue.tagBits & 
        0x1) != 0) {
        this.continueLabel = null;
      } else {
        actionInfo = actionInfo.mergedWith(loopingContext.initsOnContinue);
        loopingContext.complainOnDeferredFinalChecks(this.scope, (FlowInfo)actionInfo);
        exitBranch.addPotentialInitializationsFrom((FlowInfo)actionInfo);
      } 
    } else {
      exitBranch = condInfo.initsWhenFalse();
    } 
    boolean hasEmptyAction = !(this.action != null && 
      !this.action.isEmptyBlock() && (
      this.action.bits & 0x1) == 0);
    switch (this.kind) {
      case 0:
        if (!hasEmptyAction || 
          elementVarBinding.resolvedPosition != -1) {
          this.collectionVariable.useFlag = 1;
          if (this.continueLabel != null) {
            this.indexVariable.useFlag = 1;
            this.maxVariable.useFlag = 1;
          } 
        } 
        break;
      case 1:
      case 2:
        this.indexVariable.useFlag = 1;
        break;
    } 
    loopingContext.complainOnDeferredNullChecks(currentScope, (FlowInfo)actionInfo);
    if (loopingContext.hasEscapingExceptions()) {
      UnconditionalFlowInfo unconditionalFlowInfo;
      FlowInfo loopbackFlowInfo = flowInfo.copy();
      if (this.continueLabel != null)
        unconditionalFlowInfo = loopbackFlowInfo.mergedWith(loopbackFlowInfo.unconditionalCopy().addNullInfoFrom((FlowInfo)actionInfo).unconditionalInits()); 
      loopingContext.simulateThrowAfterLoopBack((FlowInfo)unconditionalFlowInfo);
    } 
    UnconditionalFlowInfo unconditionalFlowInfo1 = FlowInfo.mergedOptimizedBranches(
        ((loopingContext.initsOnBreak.tagBits & 
        0x3) != 0) ? 
        (FlowInfo)loopingContext.initsOnBreak : 
        flowInfo.addInitializationsFrom((FlowInfo)loopingContext.initsOnBreak), 
        false, 
        exitBranch, 
        false, 
        true);
    unconditionalFlowInfo1.resetAssignmentInfo(this.elementVariable.binding);
    this.mergedInitStateIndex = currentScope.methodScope().recordInitializationStates((FlowInfo)unconditionalFlowInfo1);
    return (FlowInfo)unconditionalFlowInfo1;
  }
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream) {
    if ((this.bits & Integer.MIN_VALUE) == 0)
      return; 
    int pc = codeStream.position;
    boolean hasEmptyAction = !(this.action != null && 
      !this.action.isEmptyBlock() && (
      this.action.bits & 0x1) == 0);
    if (hasEmptyAction && 
      this.elementVariable.binding.resolvedPosition == -1 && 
      this.kind == 0) {
      this.collection.generateCode(this.scope, codeStream, false);
      codeStream.exitUserScope(this.scope);
      if (this.mergedInitStateIndex != -1) {
        codeStream.removeNotDefinitelyAssignedVariables((Scope)currentScope, this.mergedInitStateIndex);
        codeStream.addDefinitelyAssignedVariables((Scope)currentScope, this.mergedInitStateIndex);
      } 
      codeStream.recordPositionsFrom(pc, this.sourceStart);
      return;
    } 
    switch (this.kind) {
      case 0:
        this.collection.generateCode(this.scope, codeStream, true);
        codeStream.store(this.collectionVariable, true);
        codeStream.addVariable(this.collectionVariable);
        if (this.continueLabel != null) {
          codeStream.arraylength();
          codeStream.store(this.maxVariable, false);
          codeStream.addVariable(this.maxVariable);
          codeStream.iconst_0();
          codeStream.store(this.indexVariable, false);
          codeStream.addVariable(this.indexVariable);
        } 
        break;
      case 1:
      case 2:
        this.collection.generateCode(this.scope, codeStream, true);
        codeStream.invokeIterableIterator(this.iteratorReceiverType);
        codeStream.store(this.indexVariable, false);
        codeStream.addVariable(this.indexVariable);
        break;
    } 
    BranchLabel actionLabel = new BranchLabel(codeStream);
    actionLabel.tagBits |= 0x2;
    BranchLabel conditionLabel = new BranchLabel(codeStream);
    conditionLabel.tagBits |= 0x2;
    this.breakLabel.initialize(codeStream);
    if (this.continueLabel == null) {
      conditionLabel.place();
      int conditionPC = codeStream.position;
      switch (this.kind) {
        case 0:
          codeStream.arraylength();
          codeStream.ifeq(this.breakLabel);
          break;
        case 1:
        case 2:
          codeStream.load(this.indexVariable);
          codeStream.invokeJavaUtilIteratorHasNext();
          codeStream.ifeq(this.breakLabel);
          break;
      } 
      codeStream.recordPositionsFrom(conditionPC, this.elementVariable.sourceStart);
    } else {
      this.continueLabel.initialize(codeStream);
      this.continueLabel.tagBits |= 0x2;
      codeStream.goto_(conditionLabel);
    } 
    actionLabel.place();
    switch (this.kind) {
      case 0:
        if (this.elementVariable.binding.resolvedPosition != -1) {
          codeStream.load(this.collectionVariable);
          if (this.continueLabel == null) {
            codeStream.iconst_0();
          } else {
            codeStream.load(this.indexVariable);
          } 
          codeStream.arrayAt(this.collectionElementType.id);
          if (this.elementVariableImplicitWidening != -1)
            codeStream.generateImplicitConversion(this.elementVariableImplicitWidening); 
          codeStream.store(this.elementVariable.binding, false);
          codeStream.addVisibleLocalVariable(this.elementVariable.binding);
          if (this.postCollectionInitStateIndex != -1)
            codeStream.addDefinitelyAssignedVariables(
                (Scope)currentScope, 
                this.postCollectionInitStateIndex); 
        } 
        break;
      case 1:
      case 2:
        codeStream.load(this.indexVariable);
        codeStream.invokeJavaUtilIteratorNext();
        if (this.elementVariable.binding.type.id != 1)
          if (this.elementVariableImplicitWidening != -1) {
            codeStream.checkcast(this.collectionElementType);
            codeStream.generateImplicitConversion(this.elementVariableImplicitWidening);
          } else {
            codeStream.checkcast(this.elementVariable.binding.type);
          }  
        if (this.elementVariable.binding.resolvedPosition == -1) {
          switch (this.elementVariable.binding.type.id) {
            case 7:
            case 8:
              codeStream.pop2();
              break;
          } 
          codeStream.pop();
          break;
        } 
        codeStream.store(this.elementVariable.binding, false);
        codeStream.addVisibleLocalVariable(this.elementVariable.binding);
        if (this.postCollectionInitStateIndex != -1)
          codeStream.addDefinitelyAssignedVariables(
              (Scope)currentScope, 
              this.postCollectionInitStateIndex); 
        break;
    } 
    if (!hasEmptyAction)
      this.action.generateCode(this.scope, codeStream); 
    codeStream.removeVariable(this.elementVariable.binding);
    if (this.postCollectionInitStateIndex != -1)
      codeStream.removeNotDefinitelyAssignedVariables((Scope)currentScope, this.postCollectionInitStateIndex); 
    if (this.continueLabel != null) {
      this.continueLabel.place();
      int continuationPC = codeStream.position;
      switch (this.kind) {
        case 0:
          if (!hasEmptyAction || this.elementVariable.binding.resolvedPosition >= 0)
            codeStream.iinc(this.indexVariable.resolvedPosition, 1); 
          conditionLabel.place();
          codeStream.load(this.indexVariable);
          codeStream.load(this.maxVariable);
          codeStream.if_icmplt(actionLabel);
          break;
        case 1:
        case 2:
          conditionLabel.place();
          codeStream.load(this.indexVariable);
          codeStream.invokeJavaUtilIteratorHasNext();
          codeStream.ifne(actionLabel);
          break;
      } 
      codeStream.recordPositionsFrom(continuationPC, this.elementVariable.sourceStart);
    } 
    switch (this.kind) {
      case 0:
        codeStream.removeVariable(this.indexVariable);
        codeStream.removeVariable(this.maxVariable);
        codeStream.removeVariable(this.collectionVariable);
        break;
      case 1:
      case 2:
        codeStream.removeVariable(this.indexVariable);
        break;
    } 
    codeStream.exitUserScope(this.scope);
    if (this.mergedInitStateIndex != -1) {
      codeStream.removeNotDefinitelyAssignedVariables((Scope)currentScope, this.mergedInitStateIndex);
      codeStream.addDefinitelyAssignedVariables((Scope)currentScope, this.mergedInitStateIndex);
    } 
    this.breakLabel.place();
    codeStream.recordPositionsFrom(pc, this.sourceStart);
  }
  
  public StringBuffer printStatement(int indent, StringBuffer output) {
    printIndent(indent, output).append("for (");
    this.elementVariable.printAsExpression(0, output);
    output.append(" : ");
    if (this.collection != null) {
      this.collection.print(0, output).append(") ");
    } else {
      output.append(')');
    } 
    if (this.action == null) {
      output.append(';');
    } else {
      output.append('\n');
      this.action.printStatement(indent + 1, output);
    } 
    return output;
  }
  
  public void resolve(BlockScope upperScope) {
    // Byte code:
    //   0: aload_0
    //   1: new org/eclipse/jdt/internal/compiler/lookup/BlockScope
    //   4: dup
    //   5: aload_1
    //   6: invokespecial <init> : (Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;)V
    //   9: putfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   12: aload_0
    //   13: getfield elementVariable : Lorg/eclipse/jdt/internal/compiler/ast/LocalDeclaration;
    //   16: aload_0
    //   17: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   20: invokevirtual resolve : (Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;)V
    //   23: aload_0
    //   24: getfield elementVariable : Lorg/eclipse/jdt/internal/compiler/ast/LocalDeclaration;
    //   27: getfield type : Lorg/eclipse/jdt/internal/compiler/ast/TypeReference;
    //   30: getfield resolvedType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   33: astore_2
    //   34: aload_0
    //   35: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   38: ifnonnull -> 45
    //   41: aconst_null
    //   42: goto -> 53
    //   45: aload_0
    //   46: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   49: aload_1
    //   50: invokevirtual resolveType : (Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;)Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   53: astore_3
    //   54: aconst_null
    //   55: astore #4
    //   57: aload_2
    //   58: ifnull -> 1242
    //   61: aload_3
    //   62: ifnull -> 1242
    //   65: aload_0
    //   66: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   69: invokevirtual compilerOptions : ()Lorg/eclipse/jdt/internal/compiler/impl/CompilerOptions;
    //   72: getfield targetJDK : J
    //   75: ldc2_w 3145728
    //   78: lcmp
    //   79: ifne -> 86
    //   82: iconst_1
    //   83: goto -> 87
    //   86: iconst_0
    //   87: istore #5
    //   89: aload_3
    //   90: invokevirtual isCapture : ()Z
    //   93: ifeq -> 121
    //   96: aload_3
    //   97: checkcast org/eclipse/jdt/internal/compiler/lookup/CaptureBinding
    //   100: getfield firstBound : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   103: astore #6
    //   105: aload #6
    //   107: ifnull -> 121
    //   110: aload #6
    //   112: invokevirtual isArrayType : ()Z
    //   115: ifeq -> 121
    //   118: aload #6
    //   120: astore_3
    //   121: aload_3
    //   122: invokevirtual isArrayType : ()Z
    //   125: ifeq -> 458
    //   128: aload_0
    //   129: iconst_0
    //   130: putfield kind : I
    //   133: aload_0
    //   134: aload_3
    //   135: checkcast org/eclipse/jdt/internal/compiler/lookup/ArrayBinding
    //   138: invokevirtual elementsType : ()Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   141: putfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   144: aload_0
    //   145: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   148: aload_2
    //   149: invokevirtual isCompatibleWith : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Z
    //   152: ifne -> 192
    //   155: aload_0
    //   156: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   159: aload_0
    //   160: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   163: aload_2
    //   164: invokevirtual isBoxingCompatibleWith : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Z
    //   167: ifne -> 192
    //   170: aload_0
    //   171: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   174: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   177: aload_0
    //   178: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   181: aload_0
    //   182: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   185: aload_2
    //   186: invokevirtual notCompatibleTypesErrorInForeach : (Lorg/eclipse/jdt/internal/compiler/ast/Expression;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   189: goto -> 222
    //   192: aload_0
    //   193: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   196: aload_2
    //   197: invokevirtual needsUncheckedConversion : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Z
    //   200: ifeq -> 222
    //   203: aload_0
    //   204: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   207: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   210: aload_0
    //   211: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   214: aload_0
    //   215: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   218: aload_2
    //   219: invokevirtual unsafeElementTypeConversion : (Lorg/eclipse/jdt/internal/compiler/ast/Expression;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   222: aload_0
    //   223: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   226: getfield id : I
    //   229: istore #6
    //   231: aload_2
    //   232: invokevirtual isBaseType : ()Z
    //   235: ifeq -> 350
    //   238: aload_0
    //   239: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   242: aload_0
    //   243: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   246: aload_3
    //   247: aload_3
    //   248: invokevirtual computeConversion : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   251: aload_0
    //   252: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   255: invokevirtual isBaseType : ()Z
    //   258: ifne -> 334
    //   261: aload_0
    //   262: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   265: invokevirtual environment : ()Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;
    //   268: aload_0
    //   269: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   272: invokevirtual computeBoxingType : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   275: getfield id : I
    //   278: istore #6
    //   280: aload_0
    //   281: sipush #1024
    //   284: putfield elementVariableImplicitWidening : I
    //   287: aload_2
    //   288: invokevirtual isBaseType : ()Z
    //   291: ifeq -> 980
    //   294: aload_0
    //   295: dup
    //   296: getfield elementVariableImplicitWidening : I
    //   299: aload_2
    //   300: getfield id : I
    //   303: iconst_4
    //   304: ishl
    //   305: iload #6
    //   307: iadd
    //   308: ior
    //   309: putfield elementVariableImplicitWidening : I
    //   312: aload_0
    //   313: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   316: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   319: aload_0
    //   320: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   323: aload_0
    //   324: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   327: aload_2
    //   328: invokevirtual autoboxing : (Lorg/eclipse/jdt/internal/compiler/ast/Expression;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   331: goto -> 980
    //   334: aload_0
    //   335: aload_2
    //   336: getfield id : I
    //   339: iconst_4
    //   340: ishl
    //   341: iload #6
    //   343: iadd
    //   344: putfield elementVariableImplicitWidening : I
    //   347: goto -> 980
    //   350: aload_0
    //   351: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   354: invokevirtual isBaseType : ()Z
    //   357: ifeq -> 433
    //   360: aload_0
    //   361: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   364: aload_0
    //   365: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   368: aload_3
    //   369: aload_3
    //   370: invokevirtual computeConversion : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   373: aload_0
    //   374: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   377: invokevirtual environment : ()Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;
    //   380: aload_0
    //   381: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   384: invokevirtual computeBoxingType : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   387: getfield id : I
    //   390: istore #7
    //   392: aload_0
    //   393: sipush #512
    //   396: iload #6
    //   398: iconst_4
    //   399: ishl
    //   400: ior
    //   401: iload #6
    //   403: ior
    //   404: putfield elementVariableImplicitWidening : I
    //   407: iload #7
    //   409: istore #6
    //   411: aload_0
    //   412: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   415: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   418: aload_0
    //   419: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   422: aload_0
    //   423: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   426: aload_2
    //   427: invokevirtual autoboxing : (Lorg/eclipse/jdt/internal/compiler/ast/Expression;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   430: goto -> 980
    //   433: aload_1
    //   434: aload_2
    //   435: iconst_1
    //   436: invokevirtual createArrayType : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;I)Lorg/eclipse/jdt/internal/compiler/lookup/ArrayBinding;
    //   439: astore #4
    //   441: aload_0
    //   442: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   445: aload_0
    //   446: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   449: aload #4
    //   451: aload_3
    //   452: invokevirtual computeConversion : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   455: goto -> 980
    //   458: aload_3
    //   459: instanceof org/eclipse/jdt/internal/compiler/lookup/ReferenceBinding
    //   462: ifeq -> 980
    //   465: aload_3
    //   466: checkcast org/eclipse/jdt/internal/compiler/lookup/ReferenceBinding
    //   469: bipush #38
    //   471: iconst_0
    //   472: invokevirtual findSuperTypeOriginatingFrom : (IZ)Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   475: astore #6
    //   477: aload #6
    //   479: ifnonnull -> 499
    //   482: iload #5
    //   484: ifeq -> 499
    //   487: aload_3
    //   488: checkcast org/eclipse/jdt/internal/compiler/lookup/ReferenceBinding
    //   491: bipush #59
    //   493: iconst_0
    //   494: invokevirtual findSuperTypeOriginatingFrom : (IZ)Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   497: astore #6
    //   499: aload #6
    //   501: ifnonnull -> 507
    //   504: goto -> 980
    //   507: aload_0
    //   508: aload_3
    //   509: invokevirtual erasure : ()Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   512: putfield iteratorReceiverType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   515: iload #5
    //   517: ifeq -> 575
    //   520: aload_0
    //   521: getfield iteratorReceiverType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   524: checkcast org/eclipse/jdt/internal/compiler/lookup/ReferenceBinding
    //   527: bipush #59
    //   529: iconst_0
    //   530: invokevirtual findSuperTypeOriginatingFrom : (IZ)Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   533: ifnonnull -> 559
    //   536: aload_0
    //   537: aload #6
    //   539: putfield iteratorReceiverType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   542: aload_0
    //   543: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   546: aload_0
    //   547: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   550: aload #6
    //   552: aload_3
    //   553: invokevirtual computeConversion : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   556: goto -> 627
    //   559: aload_0
    //   560: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   563: aload_0
    //   564: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   567: aload_3
    //   568: aload_3
    //   569: invokevirtual computeConversion : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   572: goto -> 627
    //   575: aload_0
    //   576: getfield iteratorReceiverType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   579: checkcast org/eclipse/jdt/internal/compiler/lookup/ReferenceBinding
    //   582: bipush #38
    //   584: iconst_0
    //   585: invokevirtual findSuperTypeOriginatingFrom : (IZ)Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   588: ifnonnull -> 614
    //   591: aload_0
    //   592: aload #6
    //   594: putfield iteratorReceiverType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   597: aload_0
    //   598: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   601: aload_0
    //   602: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   605: aload #6
    //   607: aload_3
    //   608: invokevirtual computeConversion : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   611: goto -> 627
    //   614: aload_0
    //   615: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   618: aload_0
    //   619: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   622: aload_3
    //   623: aload_3
    //   624: invokevirtual computeConversion : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   627: aconst_null
    //   628: astore #7
    //   630: aload #6
    //   632: invokevirtual kind : ()I
    //   635: lookupswitch default -> 755, 260 -> 742, 1028 -> 668, 2052 -> 732
    //   668: aload_0
    //   669: iconst_1
    //   670: putfield kind : I
    //   673: aload_0
    //   674: aload_0
    //   675: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   678: invokevirtual getJavaLangObject : ()Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   681: putfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   684: aload_0
    //   685: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   688: aload_2
    //   689: invokevirtual isCompatibleWith : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Z
    //   692: ifne -> 980
    //   695: aload_0
    //   696: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   699: aload_0
    //   700: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   703: aload_2
    //   704: invokevirtual isBoxingCompatibleWith : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Z
    //   707: ifne -> 980
    //   710: aload_0
    //   711: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   714: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   717: aload_0
    //   718: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   721: aload_0
    //   722: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   725: aload_2
    //   726: invokevirtual notCompatibleTypesErrorInForeach : (Lorg/eclipse/jdt/internal/compiler/ast/Expression;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   729: goto -> 980
    //   732: aload #6
    //   734: invokevirtual typeVariables : ()[Lorg/eclipse/jdt/internal/compiler/lookup/TypeVariableBinding;
    //   737: astore #7
    //   739: goto -> 758
    //   742: aload #6
    //   744: checkcast org/eclipse/jdt/internal/compiler/lookup/ParameterizedTypeBinding
    //   747: getfield arguments : [Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   750: astore #7
    //   752: goto -> 758
    //   755: goto -> 980
    //   758: aload #7
    //   760: arraylength
    //   761: iconst_1
    //   762: if_icmpeq -> 768
    //   765: goto -> 980
    //   768: aload_0
    //   769: iconst_2
    //   770: putfield kind : I
    //   773: aload_0
    //   774: aload #7
    //   776: iconst_0
    //   777: aaload
    //   778: putfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   781: aload_0
    //   782: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   785: aload_2
    //   786: invokevirtual isCompatibleWith : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Z
    //   789: ifne -> 829
    //   792: aload_0
    //   793: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   796: aload_0
    //   797: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   800: aload_2
    //   801: invokevirtual isBoxingCompatibleWith : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Z
    //   804: ifne -> 829
    //   807: aload_0
    //   808: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   811: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   814: aload_0
    //   815: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   818: aload_0
    //   819: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   822: aload_2
    //   823: invokevirtual notCompatibleTypesErrorInForeach : (Lorg/eclipse/jdt/internal/compiler/ast/Expression;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   826: goto -> 859
    //   829: aload_0
    //   830: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   833: aload_2
    //   834: invokevirtual needsUncheckedConversion : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Z
    //   837: ifeq -> 859
    //   840: aload_0
    //   841: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   844: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   847: aload_0
    //   848: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   851: aload_0
    //   852: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   855: aload_2
    //   856: invokevirtual unsafeElementTypeConversion : (Lorg/eclipse/jdt/internal/compiler/ast/Expression;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)V
    //   859: aload_0
    //   860: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   863: getfield id : I
    //   866: istore #8
    //   868: aload_2
    //   869: invokevirtual isBaseType : ()Z
    //   872: ifeq -> 955
    //   875: aload_0
    //   876: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   879: invokevirtual isBaseType : ()Z
    //   882: ifne -> 939
    //   885: aload_0
    //   886: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   889: invokevirtual environment : ()Lorg/eclipse/jdt/internal/compiler/lookup/LookupEnvironment;
    //   892: aload_0
    //   893: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   896: invokevirtual computeBoxingType : (Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;)Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   899: getfield id : I
    //   902: istore #8
    //   904: aload_0
    //   905: sipush #1024
    //   908: putfield elementVariableImplicitWidening : I
    //   911: aload_2
    //   912: invokevirtual isBaseType : ()Z
    //   915: ifeq -> 980
    //   918: aload_0
    //   919: dup
    //   920: getfield elementVariableImplicitWidening : I
    //   923: aload_2
    //   924: getfield id : I
    //   927: iconst_4
    //   928: ishl
    //   929: iload #8
    //   931: iadd
    //   932: ior
    //   933: putfield elementVariableImplicitWidening : I
    //   936: goto -> 980
    //   939: aload_0
    //   940: aload_2
    //   941: getfield id : I
    //   944: iconst_4
    //   945: ishl
    //   946: iload #8
    //   948: iadd
    //   949: putfield elementVariableImplicitWidening : I
    //   952: goto -> 980
    //   955: aload_0
    //   956: getfield collectionElementType : Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   959: invokevirtual isBaseType : ()Z
    //   962: ifeq -> 980
    //   965: aload_0
    //   966: sipush #512
    //   969: iload #8
    //   971: iconst_4
    //   972: ishl
    //   973: ior
    //   974: iload #8
    //   976: ior
    //   977: putfield elementVariableImplicitWidening : I
    //   980: aload_0
    //   981: getfield kind : I
    //   984: tableswitch default -> 1206, 0 -> 1012, 1 -> 1159, 2 -> 1159
    //   1012: aload_0
    //   1013: new org/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding
    //   1016: dup
    //   1017: getstatic org/eclipse/jdt/internal/compiler/ast/ForeachStatement.SecretIndexVariableName : [C
    //   1020: getstatic org/eclipse/jdt/internal/compiler/lookup/TypeBinding.INT : Lorg/eclipse/jdt/internal/compiler/lookup/BaseTypeBinding;
    //   1023: iconst_0
    //   1024: iconst_0
    //   1025: invokespecial <init> : ([CLorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;IZ)V
    //   1028: putfield indexVariable : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   1031: aload_0
    //   1032: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   1035: aload_0
    //   1036: getfield indexVariable : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   1039: invokevirtual addLocalVariable : (Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;)V
    //   1042: aload_0
    //   1043: getfield indexVariable : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   1046: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   1049: invokevirtual setConstant : (Lorg/eclipse/jdt/internal/compiler/impl/Constant;)V
    //   1052: aload_0
    //   1053: new org/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding
    //   1056: dup
    //   1057: getstatic org/eclipse/jdt/internal/compiler/ast/ForeachStatement.SecretMaxVariableName : [C
    //   1060: getstatic org/eclipse/jdt/internal/compiler/lookup/TypeBinding.INT : Lorg/eclipse/jdt/internal/compiler/lookup/BaseTypeBinding;
    //   1063: iconst_0
    //   1064: iconst_0
    //   1065: invokespecial <init> : ([CLorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;IZ)V
    //   1068: putfield maxVariable : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   1071: aload_0
    //   1072: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   1075: aload_0
    //   1076: getfield maxVariable : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   1079: invokevirtual addLocalVariable : (Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;)V
    //   1082: aload_0
    //   1083: getfield maxVariable : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   1086: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   1089: invokevirtual setConstant : (Lorg/eclipse/jdt/internal/compiler/impl/Constant;)V
    //   1092: aload #4
    //   1094: ifnonnull -> 1117
    //   1097: aload_0
    //   1098: new org/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding
    //   1101: dup
    //   1102: getstatic org/eclipse/jdt/internal/compiler/ast/ForeachStatement.SecretCollectionVariableName : [C
    //   1105: aload_3
    //   1106: iconst_0
    //   1107: iconst_0
    //   1108: invokespecial <init> : ([CLorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;IZ)V
    //   1111: putfield collectionVariable : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   1114: goto -> 1135
    //   1117: aload_0
    //   1118: new org/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding
    //   1121: dup
    //   1122: getstatic org/eclipse/jdt/internal/compiler/ast/ForeachStatement.SecretCollectionVariableName : [C
    //   1125: aload #4
    //   1127: iconst_0
    //   1128: iconst_0
    //   1129: invokespecial <init> : ([CLorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;IZ)V
    //   1132: putfield collectionVariable : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   1135: aload_0
    //   1136: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   1139: aload_0
    //   1140: getfield collectionVariable : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   1143: invokevirtual addLocalVariable : (Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;)V
    //   1146: aload_0
    //   1147: getfield collectionVariable : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   1150: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   1153: invokevirtual setConstant : (Lorg/eclipse/jdt/internal/compiler/impl/Constant;)V
    //   1156: goto -> 1242
    //   1159: aload_0
    //   1160: new org/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding
    //   1163: dup
    //   1164: getstatic org/eclipse/jdt/internal/compiler/ast/ForeachStatement.SecretIteratorVariableName : [C
    //   1167: aload_0
    //   1168: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   1171: invokevirtual getJavaUtilIterator : ()Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   1174: iconst_0
    //   1175: iconst_0
    //   1176: invokespecial <init> : ([CLorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;IZ)V
    //   1179: putfield indexVariable : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   1182: aload_0
    //   1183: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   1186: aload_0
    //   1187: getfield indexVariable : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   1190: invokevirtual addLocalVariable : (Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;)V
    //   1193: aload_0
    //   1194: getfield indexVariable : Lorg/eclipse/jdt/internal/compiler/lookup/LocalVariableBinding;
    //   1197: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   1200: invokevirtual setConstant : (Lorg/eclipse/jdt/internal/compiler/impl/Constant;)V
    //   1203: goto -> 1242
    //   1206: iload #5
    //   1208: ifeq -> 1228
    //   1211: aload_0
    //   1212: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   1215: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   1218: aload_0
    //   1219: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   1222: invokevirtual invalidTypeForCollectionTarget14 : (Lorg/eclipse/jdt/internal/compiler/ast/Expression;)V
    //   1225: goto -> 1242
    //   1228: aload_0
    //   1229: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   1232: invokevirtual problemReporter : ()Lorg/eclipse/jdt/internal/compiler/problem/ProblemReporter;
    //   1235: aload_0
    //   1236: getfield collection : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   1239: invokevirtual invalidTypeForCollection : (Lorg/eclipse/jdt/internal/compiler/ast/Expression;)V
    //   1242: aload_0
    //   1243: getfield action : Lorg/eclipse/jdt/internal/compiler/ast/Statement;
    //   1246: ifnull -> 1260
    //   1249: aload_0
    //   1250: getfield action : Lorg/eclipse/jdt/internal/compiler/ast/Statement;
    //   1253: aload_0
    //   1254: getfield scope : Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   1257: invokevirtual resolve : (Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;)V
    //   1260: return
    // Line number table:
    //   Java source line number -> byte code offset
    //   #419	-> 0
    //   #420	-> 12
    //   #421	-> 23
    //   #422	-> 34
    //   #424	-> 54
    //   #425	-> 57
    //   #426	-> 65
    //   #427	-> 89
    //   #428	-> 96
    //   #429	-> 105
    //   #430	-> 118
    //   #432	-> 121
    //   #433	-> 128
    //   #434	-> 133
    //   #435	-> 144
    //   #436	-> 155
    //   #437	-> 170
    //   #438	-> 189
    //   #439	-> 203
    //   #442	-> 222
    //   #443	-> 231
    //   #444	-> 238
    //   #445	-> 251
    //   #446	-> 261
    //   #447	-> 280
    //   #448	-> 287
    //   #449	-> 294
    //   #450	-> 312
    //   #452	-> 331
    //   #453	-> 334
    //   #455	-> 347
    //   #456	-> 360
    //   #457	-> 373
    //   #458	-> 392
    //   #459	-> 407
    //   #460	-> 411
    //   #461	-> 430
    //   #462	-> 433
    //   #463	-> 441
    //   #465	-> 455
    //   #466	-> 465
    //   #467	-> 477
    //   #468	-> 487
    //   #471	-> 499
    //   #473	-> 507
    //   #474	-> 515
    //   #475	-> 520
    //   #476	-> 536
    //   #477	-> 542
    //   #478	-> 556
    //   #479	-> 559
    //   #481	-> 572
    //   #482	-> 591
    //   #483	-> 597
    //   #484	-> 611
    //   #485	-> 614
    //   #488	-> 627
    //   #489	-> 630
    //   #491	-> 668
    //   #492	-> 673
    //   #493	-> 684
    //   #494	-> 695
    //   #495	-> 710
    //   #498	-> 729
    //   #501	-> 732
    //   #502	-> 739
    //   #505	-> 742
    //   #506	-> 752
    //   #509	-> 755
    //   #512	-> 758
    //   #513	-> 768
    //   #515	-> 773
    //   #516	-> 781
    //   #517	-> 792
    //   #518	-> 807
    //   #519	-> 826
    //   #520	-> 840
    //   #522	-> 859
    //   #524	-> 868
    //   #525	-> 875
    //   #526	-> 885
    //   #527	-> 904
    //   #528	-> 911
    //   #529	-> 918
    //   #531	-> 936
    //   #532	-> 939
    //   #534	-> 952
    //   #535	-> 955
    //   #536	-> 965
    //   #541	-> 980
    //   #544	-> 1012
    //   #545	-> 1031
    //   #546	-> 1042
    //   #548	-> 1052
    //   #549	-> 1071
    //   #550	-> 1082
    //   #552	-> 1092
    //   #553	-> 1097
    //   #554	-> 1114
    //   #555	-> 1117
    //   #557	-> 1135
    //   #558	-> 1146
    //   #559	-> 1156
    //   #563	-> 1159
    //   #564	-> 1182
    //   #565	-> 1193
    //   #566	-> 1203
    //   #568	-> 1206
    //   #569	-> 1211
    //   #570	-> 1225
    //   #571	-> 1228
    //   #575	-> 1242
    //   #576	-> 1249
    //   #578	-> 1260
    // Local variable table:
    //   start	length	slot	name	descriptor
    //   0	1261	0	this	Lorg/eclipse/jdt/internal/compiler/ast/ForeachStatement;
    //   0	1261	1	upperScope	Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   34	1227	2	elementType	Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   54	1207	3	collectionType	Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   57	1204	4	expectedCollectionType	Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   89	1153	5	isTargetJsr14	Z
    //   105	16	6	upperBound	Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   231	224	6	compileTimeTypeID	I
    //   392	38	7	boxedID	I
    //   477	503	6	iterableType	Lorg/eclipse/jdt/internal/compiler/lookup/ReferenceBinding;
    //   630	350	7	arguments	[Lorg/eclipse/jdt/internal/compiler/lookup/TypeBinding;
    //   868	112	8	compileTimeTypeID	I
  }
  
  public void traverse(ASTVisitor visitor, BlockScope blockScope) {
    if (visitor.visit(this, blockScope)) {
      this.elementVariable.traverse(visitor, this.scope);
      if (this.collection != null)
        this.collection.traverse(visitor, this.scope); 
      if (this.action != null)
        this.action.traverse(visitor, this.scope); 
    } 
    visitor.endVisit(this, blockScope);
  }
  
  public boolean doesNotCompleteNormally() {
    return false;
  }
}
